package org.apache.geronimo.gshell.maven.javacc;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

/**
 * Display help information on 'org.apache.geronimo.gshell:javacc-maven-plugin' plugin. Call 'mvn javacc:help -Ddetail=true' to display all details.
 *
 * @version generated on Thu May 08 23:54:17 CEST 2008
 * @goal help
 */
public class HelpMojo
    extends AbstractMojo
{
    /** 80-character display buffer */
    private static final int DEFAULT_WIDTH = 80;

    /** 4 indent spaces */
    private static final String DEFAULT_INDENT = repeat( " ", 2 );

    /**
     * If true, display all settable properies for each goal.
     *
     * @parameter expression="${detail}" default-value="false"
     */
    private boolean detail;

    /** {@inheritDoc} */
    public void execute()
        throws MojoExecutionException
    {
        StringBuffer sb = new StringBuffer();

        sb.append( "The 'org.apache.geronimo.gshell:javacc-maven-plugin' plugin has 3 goals:" ).append( "\n" );
        sb.append( "\n" );

        sb.append( "javacc:jjtree" ).append( "\n" );
        for ( Iterator it = toLines( "Invokes the JJTree AST generator pre-processor." ).iterator(); it.hasNext(); )
        {
            sb.append( it.next().toString() ).append( "\n" );
        }
        if ( detail )
        {
            sb.append( "\n" );

            sb.append( repeat( " ", 2 ) );
            sb.append( "Available parameters:" ).append( "\n" );

            sb.append( "\n" );

        for ( Iterator it = toLines( "grammars", repeat( " ", 4 ), DEFAULT_WIDTH ).iterator(); it.hasNext(); )
        {
            sb.append( it.next().toString() ).append( "\n" );
        }
            for ( Iterator it = toLines( "A set of grammar files to process.", repeat( " ", 6 ), DEFAULT_WIDTH ).iterator(); it.hasNext(); )
            {
                sb.append( it.next().toString() ).append( "\n" );
            }
        for ( Iterator it = toLines( "nodePackage", repeat( " ", 4 ), DEFAULT_WIDTH ).iterator(); it.hasNext(); )
        {
            sb.append( it.next().toString() ).append( "\n" );
        }
            for ( Iterator it = toLines( "The package for AST node classes.", repeat( " ", 6 ), DEFAULT_WIDTH ).iterator(); it.hasNext(); )
            {
                sb.append( it.next().toString() ).append( "\n" );
            }
        for ( Iterator it = toLines( "outputDirectory", repeat( " ", 4 ), DEFAULT_WIDTH ).iterator(); it.hasNext(); )
        {
            sb.append( it.next().toString() ).append( "\n" );
        }
            for ( Iterator it = toLines( "Where to output generated sources.", repeat( " ", 6 ), DEFAULT_WIDTH ).iterator(); it.hasNext(); )
            {
                sb.append( it.next().toString() ).append( "\n" );
            }
        }

        sb.append( "\n" );

        sb.append( "javacc:javacc" ).append( "\n" );
        for ( Iterator it = toLines( "Invokes the JavaCC compiler compiler." ).iterator(); it.hasNext(); )
        {
            sb.append( it.next().toString() ).append( "\n" );
        }
        if ( detail )
        {
            sb.append( "\n" );

            sb.append( repeat( " ", 2 ) );
            sb.append( "Available parameters:" ).append( "\n" );

            sb.append( "\n" );

        for ( Iterator it = toLines( "grammars", repeat( " ", 4 ), DEFAULT_WIDTH ).iterator(); it.hasNext(); )
        {
            sb.append( it.next().toString() ).append( "\n" );
        }
            for ( Iterator it = toLines( "A set of grammar files to process.", repeat( " ", 6 ), DEFAULT_WIDTH ).iterator(); it.hasNext(); )
            {
                sb.append( it.next().toString() ).append( "\n" );
            }
        for ( Iterator it = toLines( "outputDirectory", repeat( " ", 4 ), DEFAULT_WIDTH ).iterator(); it.hasNext(); )
        {
            sb.append( it.next().toString() ).append( "\n" );
        }
            for ( Iterator it = toLines( "Where to output generated sources.", repeat( " ", 6 ), DEFAULT_WIDTH ).iterator(); it.hasNext(); )
            {
                sb.append( it.next().toString() ).append( "\n" );
            }
        }

        sb.append( "\n" );

        sb.append( "javacc:help" ).append( "\n" );
        for ( Iterator it = toLines( "Display help information on 'org.apache.geronimo.gshell:javacc-maven-plugin' plugin. Call 'mvn javacc:help -Ddetail=true' to display all details." ).iterator(); it.hasNext(); )
        {
            sb.append( it.next().toString() ).append( "\n" );
        }

        if ( getLog().isInfoEnabled() )
        {
            getLog().info( sb.toString() );
        }
    }

    /**
     * <p>Repeat a String <code>n</code> times to form a new string.</p>
     *
     * @param str String to repeat
     * @param repeat number of times to repeat str
     * @return String with repeated String
     * @throws NegativeArraySizeException if <code>repeat < 0</code>
     * @throws NullPointerException if str is <code>null</code>
     */
    private static String repeat( String str, int repeat )
    {
        StringBuffer buffer = new StringBuffer( repeat * str.length() );

        for ( int i = 0; i < repeat; i++ )
        {
            buffer.append( str );
        }

        return buffer.toString();
    }

    /**
     * <p>Give a list of lines for the <code>str</code>. Each line is indented by 4 spaces
     * and has a maximum of <code>80</code> characters.</p>
     *
     * @param str String to split in lines
     * @return List of lines
     * @throws NullPointerException if str is <code>null</code>
     */
    private static List toLines( String str )
    {
        return toLines( str, DEFAULT_INDENT, DEFAULT_WIDTH );
    }

    /**
     * <p>Give a list of lines for the <code>str</code>. Each line is indented by <code>indent</code>
     * and has a maximum of <code>size</code> characters.</p>
     *
     * @param str String to split in lines
     * @param indent the string to precede each line
     * @param size the size of the character display buffer
     * @return List of lines
     * @throws IllegalArgumentException if <code>size < 0</code>
     * @throws NullPointerException if str is <code>null</code>
     */
    private static List toLines( String str, String indent, int size )
    {
        List sentences = new ArrayList();

        if ( indent == null )
        {
            indent = "";
        }

        if ( size < 0 )
        {
            throw new IllegalArgumentException( "size should be positive" );
        }

        StringBuffer tmp = new StringBuffer( indent );
        StringTokenizer tokenizer = new StringTokenizer( str, " " );
        while ( tokenizer.hasMoreTokens() )
        {
            String word = tokenizer.nextToken();

            if ( tmp.length() + word.length() + 1 < size )
            {
                tmp.append( word ).append( " " );
            }
            else
            {
                sentences.add( tmp.toString() );
                tmp = new StringBuffer( indent );
                tmp.append( word ).append( " " );
            }
        }

        if ( tmp.toString().length() > 0 )
        {
            sentences.add( tmp.toString() );
        }

        return sentences;
    }
}
